using System.Security.Cryptography.X509Certificates;
using FitConnect.Models;
using FitConnect.Services.Models;
using Microsoft.Extensions.Logging;

namespace FitConnect;

/// <summary>
/// The fluent implementation for the <see cref="Sender"/> to encapsulate the FitConnect API.
/// </summary>
/// <example>
/// Reference for FluentSender
/// <code>
/// client.Sender
///     .Authenticate(clientId!, clientSecret!)
///     .CreateSubmission(new Submission { Attachments = new List() })
///     .UploadAttachments()
///     .SendSubmission(new Metadata(), new Data());
/// </code>
/// </example>
public class FluentSender : Sender {
    private OAuthAccessToken? token;


    public FluentSender(FitConnectEndpoints endpoints, X509Certificate2? certificate = null,
        ILogger? logger = null) : base(endpoints, certificate, logger) {
        PublicKey = certificate.PublicKey;
    }

    public PublicKey PublicKey { get; set; }
    public CreateSubmissionDto? NewSubmission { get; set; }
    public Submission Submission { get; set; }


    /// <summary>
    ///     Authenticates with the FitConnect API.
    /// </summary>
    /// <param name="clientId">The client id from the submission service</param>
    /// <param name="clientSecret">The client secret</param>
    /// <param name="scope">Optional scope for the credentials</param>
    /// <returns></returns>
    public FluentSender Authenticate(string clientId, string clientSecret, string? scope = null) {
        Owner.ClientId = clientId;
        Owner.ClientSecret = clientSecret;

        token ??= AuthenticateAsync(clientId, clientSecret, scope).Result;
        return this;
    }

    /// <summary>
    ///     Creates a new <see cref="Submission" /> on the FitConnect server.
    /// </summary>
    /// <param name="submission" cref="Submission">The <see cref="Submission"/> body to be sent</param>
    /// <returns></returns>
    /// <exception cref="InvalidOperationException">If sender is not authenticated</exception>
    /// <exception cref="ArgumentException">If submission is not ready to be sent</exception>
    public FluentSender CreateSubmission(Submission submission) {
        Submission = submission;
        if (token == null)
            throw new InvalidOperationException("You must authenticate first.");

        if (!submission.IsSubmissionReadyToAdd(out var errorMessage))
            throw new ArgumentException($"Submission is not ready to add. {errorMessage}");

        NewSubmission = AddSubmission(submission);

        return this;
    }

    /// <summary>
    /// Send <see cref="Metadata"/> and <see cref="Data"/> to the FitConnect API.
    /// </summary>
    /// <returns></returns>
    /// <exception cref="NotImplementedException"></exception>
    public FluentSender SendSubmission() {
        throw new NotImplementedException();
    }

    /// <summary>
    /// Posts the <see cref="Attachment"/>s to the FitConnect API. The attachment ids are sent with creating the submission.
    /// </summary>
    /// <returns></returns>
    /// <exception cref="InvalidOperationException"></exception>
    public FluentSender UploadAttachments() {
        if (NewSubmission == null)
            throw new InvalidOperationException("You must create a submission first.");

        if (Submission.Attachments.Count == 0)
            return this;

        var encryptedAttachments = Encrypt(PublicKey, Submission.Attachments);
        UploadAttachmentsAsync(encryptedAttachments).Wait();

        return this;
    }

    /// <summary>
    /// Sending the submission and finishing the process.
    /// <para>
    /// Finishing the submission by transmitting <see cref="Metadata"/> and <see cref="Data"/>.
    /// </para> 
    /// </summary>
    /// <param name="metadata"></param>
    /// <param name="data"></param>
    /// <returns></returns>
    /// <remarks>This call finishes the submission process</remarks>
    /// <exception cref="InvalidOperationException"></exception>
    public FluentSender SendSubmission(Metadata metadata, Data? data = null) {
        if (NewSubmission == null)
            throw new InvalidOperationException("You must create a submission first.");

        Submission.Metadata = metadata;
        Submission.Data = data;
        var submitSubmissionDto = CreateSubmitSubmissionDto(Submission);
        ApiService.SubmissionService.SubmitSubmission(Submission.Id, submitSubmissionDto);
        return this;
    }
}
